# -*- coding: utf-8 -*-
'''
This file is part of PyMbs.

PyMbs is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

PyMbs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with Foobar.
If not, see <http://www.gnu.org/licenses/>.

Copyright 2008 Carsten Knoll, Christian Schubert,
               Jens Frenkel, Sebastian Voigt
'''

'''
Created on 14.07.2009

@author: voigt

Description of a skidsteer loader consisting of chassis, wheels, boom, bucket
and cylinders (force elements) for moving boom and bucket as well as torque
and force interfaces for wheels and driveline.
'''

from PyMbs.Input import *

CHASSIS_FIXED = True

# Create multibody system and origin
world = MbsSystem([0, 0, -1])
world.addFrame('CS_origin')


# Parameters
r = world.addParam('r', 0.424)
w = world.addParam('w', 1.08)
t = world.addParam('t', 1.395)

m_wheel = world.addParam('m_wheel', 42.2)
cg_wheel = world.addParam('cg_wheel', [0, 0, 0])
I_wheel = world.addParam('I_wheel', [2.54, 0, 4.39, 0, 0, 2.54])


# Bodies
# Chassis
if not CHASSIS_FIXED:
    chassis = world.addBody(mass=2403.8, cg=[-0.495, 0, 0.741],
              inertia=[617.79, 0.35, 1206.98, -74.05, -0.17, 1106.6])
else:
    chassis = world.addFrame('chassis')
chassis.addFrame('CS_world')
chassis.addFrame('CS_boom', [-1.244, 0, 1.7635])
chassis.addFrame('CS_lift_cylinder', [-0.977, 0, 1.1185])
chassis.addFrame('CS_lfw', [w/2, t/2, r])
chassis.addFrame('CS_rfw', [w/2, -t/2, r])
chassis.addFrame('CS_lbw', [-w/2, t/2, r])
chassis.addFrame('CS_rbw', [-w/2, -t/2, r])

# Wheels
lfw = world.addBody(m_wheel, cg_wheel, I_wheel)
rfw = world.addBody(m_wheel, cg_wheel, I_wheel)
lbw = world.addBody(m_wheel, cg_wheel, I_wheel)
rbw = world.addBody(m_wheel, cg_wheel, I_wheel)

# Boom
boom = world.addBody(mass=280, cg=[1.512, 0, -0.749],
       inertia=[108.15, 0.12, 176.51, -55.7, -0.04, 202.27])
boom.addFrame('CS_chassis')
boom.addFrame('CS_lift_cylinder_rod', [1.2558, 0, -0.715])
boom.addFrame('CS_dump_cylinder', [2.0518, 0, -0.635])
boom.addFrame('CS_bucket', [2.134, 0, -1.575])

# Bucket
bucket = world.addBody(mass=283, cg=[0.49, 0, 0.042],
         inertia=[87.74, -0.01, 30.22, -4.37, 0, 98.25])
bucket.addFrame('CS_boom')
bucket.addFrame('CS_dump_cylinder_rod', [0.1777, 0, 0.1749])

# Lift cylinder
lift_cylinder = world.addBody(mass=44.3, cg=[0.4543, 0, -0.0019],
                inertia=[13.32, 0, 3.66, 0, 0, 16.93])
lift_cylinder.addFrame('CS_chassis')
lift_cylinder_rod = world.addBody(mass=17.5, cg=[-0.4122, 0, 0],
                    inertia=[5.25, 0, 1.36, 0, 0, 6.6])
lift_cylinder_rod.addFrame('CS_boom')

# Dump cylinder
dump_cylinder = world.addBody(mass=32, cg=[0.309, 0, 0.006],
                inertia=[3.17, 0, 1.25, 0, 0, 4.39])
dump_cylinder.addFrame('CS_boom')
dump_cylinder_rod = world.addBody(mass=12.6, cg=[-0.2721, 0, 0],
                    inertia=[1.21, 0, 0.49, 0, 0, 1.7])
dump_cylinder_rod.addFrame('CS_bucket')


# Joints
# Chassis
if not CHASSIS_FIXED:
    world.addJoint(world.CS_origin, chassis.CS_world,
                   ['Tx', 'Ty', 'Tz', 'Rx', 'Ry', 'Rz'])

# Wheels
jlf = world.addJoint(chassis.CS_lfw, lfw, 'Ry')
jrf = world.addJoint(chassis.CS_rfw, rfw, 'Ry')
jlb = world.addJoint(chassis.CS_lbw, lbw, 'Ry')
jrb = world.addJoint(chassis.CS_rbw, rbw, 'Ry')

# Boom and bucket
boom_rot = world.addJoint(chassis.CS_boom, boom.CS_chassis, 'Ry')
bucket_rot = world.addJoint(boom.CS_bucket, bucket.CS_boom, 'Ry')

# Lift cylinder
lift_rot = world.addJoint(chassis.CS_lift_cylinder,
                          lift_cylinder.CS_chassis, 'Ry')
lift_trans = world.addJoint(lift_cylinder.CS_chassis,
                            lift_cylinder_rod.CS_boom, 'Tx')

# Dump cylinder
dump_rot = world.addJoint(boom.CS_dump_cylinder, dump_cylinder.CS_boom, 'Ry')
dump_trans = world.addJoint(dump_cylinder.CS_boom,
                            dump_cylinder_rod.CS_bucket, 'Tx')

# Loops
# Synchronize both left and both right wheels
world.addLoop.Transmission(jlf, jlb, 1, 'synchronous_left')
world.addLoop.Transmission(jrf, jrb, 1, 'synchronous_light')

# Cylinder drives
world.addLoop.ThreeBarTrans(boom.CS_lift_cylinder_rod,
                            lift_cylinder_rod.CS_boom)
world.addLoop.ThreeBarTrans(bucket.CS_dump_cylinder_rod,
                            dump_cylinder_rod.CS_bucket)

# Visualisation
detail = '_high'
#detail = '_low'

world.addVisualisation.File(chassis, 'Chassis%s.STL' % detail, 1e-3)
world.addVisualisation.File(lfw, 'Radlinks%s.STL' % detail, 1e-3)
world.addVisualisation.File(rfw, 'Radrechts%s.STL' % detail, 1e-3)
world.addVisualisation.File(lbw, 'Radlinks%s.STL' % detail, 1e-3)
world.addVisualisation.File(rbw, 'Radrechts%s.STL' % detail, 1e-3)

world.addVisualisation.File(boom, 'Ausleger%s.STL' % detail, 1e-3)
world.addVisualisation.File(bucket, 'Schaufel%s.STL' % detail, 1e-3)
world.addVisualisation.File(lift_cylinder, scale=1e-3,
                            fileName='Auslegerzylinder%s.STL' % detail)
world.addVisualisation.File(lift_cylinder_rod, scale=1e-3,
                            fileName='Auslegerzylinderkolben%s.STL' % detail)
world.addVisualisation.File(dump_cylinder, 'Kippzylinder%s.STL' % detail, 1e-3)
world.addVisualisation.File(dump_cylinder_rod, scale=1e-3,
                            fileName='Kippzylinderkolben%s.STL' % detail)

# Get System Equations
world.genEquations.Recursive()
world.show()









